package com.tzld.piaoquan.tencentad.utils;

import lombok.extern.slf4j.Slf4j;
import ws.schild.jave.process.ProcessWrapper;
import ws.schild.jave.process.ffmpeg.DefaultFFMPEGLocator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

@Slf4j
public class FfmpegUtil {

    public static boolean getTargetThumbnail(String url, String targetPath) {
        try {
            try (ProcessWrapper ffmpeg = new DefaultFFMPEGLocator().createExecutor();) {
//                ffmpeg.addArgument("-i");
//                ffmpeg.addArgument(url);
//                ffmpeg.addArgument("-filter:v");
//                ffmpeg.addArgument("select='eq(pict_type,I)+gt(scene,0.11)',showinfo");
//                ffmpeg.addArgument("-vsync");
//                ffmpeg.addArgument("vfr");
//                ffmpeg.addArgument(targetPath);
                ffmpeg.addArgument("-i");
                ffmpeg.addArgument(url);
                ffmpeg.addArgument("-g");
                ffmpeg.addArgument("1"); // 设置 GOP 大小为 1
                ffmpeg.addArgument("-sc_threshold");
                ffmpeg.addArgument("100"); // 设置场景切换阈值
                ffmpeg.addArgument("-vf");
                ffmpeg.addArgument("select='eq(pict_type,I)'"); // 选择关键帧
                ffmpeg.addArgument("-vsync");
                ffmpeg.addArgument("vfr"); // 变帧率
                ffmpeg.addArgument(targetPath); // 输出文件
                ffmpeg.execute();
                try (BufferedReader br = new BufferedReader(new InputStreamReader(ffmpeg.getErrorStream()))) {
                    blockFfmpeg(br);
                }
            }
        } catch (IOException e) {
            System.out.println(e.getMessage());
            log.error("获取缩略图失败 url={}", url);
            return false;
        }
        return true;
    }

    public static void main(String[] args) {
        String url = "https://rescdn.yishihui.com/long_articles/video/8b4b2e6a-e556-4d06-8b4e-65a25654ebce";
        String targetPath = "/Users/shimeng/Desktop/img" + "/" + "111.jpg";
        String timestamp = "00:01:30.500";
        System.out.println(getTargetTimeThumbnail(url, targetPath, timestamp));
    }


    public static boolean getTargetTimeThumbnail(String url, String targetPath, String timestamp) {
        try {

            DefaultFFMPEGLocator defaultFFMPEGLocator = new DefaultFFMPEGLocator();
            String executablePath = defaultFFMPEGLocator.getExecutablePath();
            log.info("executablePath={}", executablePath);
            ProcessWrapper ffmpeg = defaultFFMPEGLocator.createExecutor();

            ffmpeg.addArgument("-ss");
            ffmpeg.addArgument(timestamp);
            ffmpeg.addArgument("-i");
            ffmpeg.addArgument(url);
            ffmpeg.addArgument("-vframes");
            ffmpeg.addArgument("1");
            ffmpeg.addArgument(targetPath);
            ffmpeg.execute();
            try (BufferedReader br = new BufferedReader(new InputStreamReader(ffmpeg.getErrorStream()))) {
                blockFfmpeg(br);
            }
        } catch (IOException e) {
            log.error("getTargetTimeThumbnail error", e);
            return false;
        } catch (Exception e) {
            log.error("getTargetTimeThumbnail error", e);
        }
        return true;
    }


    /**
     * 等待命令执行成功，退出
     *
     * @param br
     * @throws IOException
     */
    private static void blockFfmpeg(BufferedReader br) throws IOException {
        String line;
        // 该方法阻塞线程，直至合成成功
        while ((line = br.readLine()) != null) {
            doNothing(line);
        }
    }

    private static void doNothing(String line) {
        // FIXME: 2023/1/31 正式使用时注释掉此行，仅用于观察日志
        log.info(line);
    }
}
